	/*********************************************************************
	 *
	 *                  Generic Type Definitions
	 *
	 *********************************************************************
	 * FileName:        GenericTypeDefs.h
	 * Dependencies:	None
	 * Processor:       PIC18, PIC24, dsPIC, PIC32
	 * Compiler:        Microchip C18, C30, C32
	 * Company:         Microchip Technology, Inc.
	 *
	 * Software License Agreement
	 *
	 * The software supplied herewith by Microchip Technology Incorporated
	 * (the "Company") is intended and supplied to you, the Company's
	 * customer, for use solely and exclusively with products manufactured
	 * by the Company.
	 *
	 * The software is owned by the Company and/or its supplier, and is
	 * protected under applicable copyright laws. All rights are reserved.
	 * Any use in violation of the foregoing restrictions may subject the
	 * user to criminal sanctions under applicable laws, as well as to
	 * civil liability for the breach of the terms and conditions of this
	 * license.
	 *
	 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
	 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
	 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
	 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
	 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
	 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
	 *
	 *********************************************************************
	 * File Description:
	 *
	 * Change History:
	 *  Rev   Date         Description
	 *  1.1   09/11/06     Add base signed types
	 *  1.2   02/28/07	   Add QWORD, LONGLONG, QWORD_VAL
	 *  1.3   02/06/08     Add def's for PIC32
	 *  1.4   08/08/08     Remove LSB/MSB Macros, adopted by Peripheral lib
	 *  1.5   08/14/08     Simplify file header
	 ********************************************************************/

#ifndef __GENERIC_TYPE_DEFS_H_
#define __GENERIC_TYPE_DEFS_H_

#ifndef __PC__

typedef enum _BOOL { FALSE = 0, TRUE } BOOL;	// Undefined size

#ifndef NULL
#define NULL    0//((void *)0)
#endif

#define	PUBLIC                                  // Function attributes
#define PROTECTED
#define PRIVATE   static

typedef unsigned char		BYTE;				// 8-bit unsigned
typedef unsigned short int	WORD;				// 16-bit unsigned
typedef unsigned long		DWORD;				// 32-bit unsigned
typedef unsigned long long	QWORD;				// 64-bit unsigned
typedef signed char			CHAR;				// 8-bit signed
typedef signed short int	SHORT;				// 16-bit signed
typedef signed long			LONG;				// 32-bit signed
typedef signed long long	LONGLONG;			// 64-bit signed

/* Alternate definitions */
typedef void                VOID;

typedef char                CHAR8;
typedef unsigned char       UCHAR8;

/* Processor & Compiler independent, size specific definitions */
// To Do:  We need to verify the sizes on each compiler.  These
//         may be compiler specific, we should either move them
//         to "compiler.h" or #ifdef them for compiler type.
typedef signed int          INT;
typedef signed char         INT8;
typedef signed short int    INT16;
typedef signed long int     INT32;
typedef signed long long    INT64;

typedef unsigned int        UINT;
typedef unsigned char       UINT8;
typedef unsigned short int  UINT16;
typedef unsigned long int   UINT32;  // other name for 32-bit integer
typedef unsigned long long  UINT64;

typedef union _BYTE_VAL
{
	BYTE Val;
	struct
	{
		unsigned char b0:1;
		unsigned char b1:1;
		unsigned char b2:1;
		unsigned char b3:1;
		unsigned char b4:1;
		unsigned char b5:1;
		unsigned char b6:1;
		unsigned char b7:1;
	} bits;
} BYTE_VAL, BYTE_BITS;

typedef union _WORD_VAL
{
	WORD Val;
	BYTE v[2];
	struct
	{
		BYTE LB;
		BYTE HB;
	} byte;
	struct
	{
		unsigned char b0:1;
		unsigned char b1:1;
		unsigned char b2:1;
		unsigned char b3:1;
		unsigned char b4:1;
		unsigned char b5:1;
		unsigned char b6:1;
		unsigned char b7:1;
		unsigned char b8:1;
		unsigned char b9:1;
		unsigned char b10:1;
		unsigned char b11:1;
		unsigned char b12:1;
		unsigned char b13:1;
		unsigned char b14:1;
		unsigned char b15:1;
	} bits;
} WORD_VAL, WORD_BITS;

typedef union _DWORD_VAL
{
	DWORD Val;
	WORD w[2];
	BYTE v[4];
	struct
	{
		WORD LW;
		WORD HW;
	} word;
	struct
	{
		BYTE LB;
		BYTE HB;
		BYTE UB;
		BYTE MB;
	} byte;
	struct
	{
		WORD_VAL low;
		WORD_VAL high;
	}wordUnion;
	struct
	{
		unsigned char b0:1;
		unsigned char b1:1;
		unsigned char b2:1;
		unsigned char b3:1;
		unsigned char b4:1;
		unsigned char b5:1;
		unsigned char b6:1;
		unsigned char b7:1;
		unsigned char b8:1;
		unsigned char b9:1;
		unsigned char b10:1;
		unsigned char b11:1;
		unsigned char b12:1;
		unsigned char b13:1;
		unsigned char b14:1;
		unsigned char b15:1;
		unsigned char b16:1;
		unsigned char b17:1;
		unsigned char b18:1;
		unsigned char b19:1;
		unsigned char b20:1;
		unsigned char b21:1;
		unsigned char b22:1;
		unsigned char b23:1;
		unsigned char b24:1;
		unsigned char b25:1;
		unsigned char b26:1;
		unsigned char b27:1;
		unsigned char b28:1;
		unsigned char b29:1;
		unsigned char b30:1;
		unsigned char b31:1;
	} bits;
} DWORD_VAL;

typedef union _QWORD_VAL
{
	QWORD Val;
	DWORD d[2];
	WORD w[4];
	BYTE v[8];
	struct
	{
		DWORD LD;
		DWORD HD;
	} dword;
	struct
	{
		WORD LW;
		WORD HW;
		WORD UW;
		WORD MW;
	} word;
	struct
	{
		unsigned char b0:1;
		unsigned char b1:1;
		unsigned char b2:1;
		unsigned char b3:1;
		unsigned char b4:1;
		unsigned char b5:1;
		unsigned char b6:1;
		unsigned char b7:1;
		unsigned char b8:1;
		unsigned char b9:1;
		unsigned char b10:1;
		unsigned char b11:1;
		unsigned char b12:1;
		unsigned char b13:1;
		unsigned char b14:1;
		unsigned char b15:1;
		unsigned char b16:1;
		unsigned char b17:1;
		unsigned char b18:1;
		unsigned char b19:1;
		unsigned char b20:1;
		unsigned char b21:1;
		unsigned char b22:1;
		unsigned char b23:1;
		unsigned char b24:1;
		unsigned char b25:1;
		unsigned char b26:1;
		unsigned char b27:1;
		unsigned char b28:1;
		unsigned char b29:1;
		unsigned char b30:1;
		unsigned char b31:1;
		unsigned char b32:1;
		unsigned char b33:1;
		unsigned char b34:1;
		unsigned char b35:1;
		unsigned char b36:1;
		unsigned char b37:1;
		unsigned char b38:1;
		unsigned char b39:1;
		unsigned char b40:1;
		unsigned char b41:1;
		unsigned char b42:1;
		unsigned char b43:1;
		unsigned char b44:1;
		unsigned char b45:1;
		unsigned char b46:1;
		unsigned char b47:1;
		unsigned char b48:1;
		unsigned char b49:1;
		unsigned char b50:1;
		unsigned char b51:1;
		unsigned char b52:1;
		unsigned char b53:1;
		unsigned char b54:1;
		unsigned char b55:1;
		unsigned char b56:1;
		unsigned char b57:1;
		unsigned char b58:1;
		unsigned char b59:1;
		unsigned char b60:1;
		unsigned char b61:1;
		unsigned char b62:1;
		unsigned char b63:1;
	} bits;
} QWORD_VAL;

#endif //#ifndef __PC__
#endif //__GENERIC_TYPE_DEFS_H_

