//Player.c
#include <string.h> //memcmp
#include <stdio.h>	//sprintf
#include "Player.h"
#include "playerhardware.h"
#include "integer.h"

#include "FSIO.h"
#include "FSDefs.h"

short MusicNum;			//Now Playing Musics Number, from 1
short WavsInFolder;		//Total WAV file count in current folder
short CurrentFolderIndex;		//Playing Folder, 0 means on the root, 1 is 1st CD
short MaxFolderCount;			//how many folder exists
FSFILE * Currentfo;		//Current FSFILE, playing now
unsigned long fCounter;	//current counter in the file

PLAYBACKDATA pbData;

#ifdef __PC__
PaStreamParameters outputParameters;
PaStream *stream = NULL;
PaError err;
#endif

int Player_Init(void)
{
	//remember how many folder exists.
	//(folder can be not for wav files.)

	short ret = 0;
	short TotalWavFound = 0;
	SearchRec rec;

	MaxFolderCount = 0;
	CurrentFolderIndex = 0;
	fCounter = 0;
	MusicNum = 0;
	WavsInFolder = 0;

	//tell playerhardware about playback data.
	SetPlayBackStruct(&pbData);
	Init_playdevice(&pbData);

	ret = FSInit();
	if (ret == FALSE) return -3;

	//Search Folders, store to Folder structure
	ret = FSchdir("\\");	//ch to root
	if (ret != 0) return -4;

	//lookup every folder.
	ret = FSFindFirst("*", ATTR_DIRECTORY, &rec);
	if (ret != 0) return -5;

	//First Folder found. trace every folder.
	ret = Player_FolderInit(rec.filename);
	FSchdir("\\"); //FolderInit will not chdir. back to root.
	MaxFolderCount++ ;
	TotalWavFound += ret;
    while (!FSFindNext( &rec ))
    {
		ret = Player_FolderInit(rec.filename);
		FSchdir("\\"); //FolderInit will not chdir. back to root.
		MaxFolderCount++ ;
		TotalWavFound += ret;
    }

	if (TotalWavFound == 0) return -2; // no wav found

	//goto 1st CD
	if (MaxFolderCount > 0) CurrentFolderIndex = 1;
	return 0;
}

int Player_FolderInit(char * foldername)
{
	int ret;
	SearchRec rec;
    unsigned char attributes;

	WavsInFolder = 0;
	
	ret = FSchdir(foldername);	//ch to folder
	if (ret != 0) return -1;

	//lookup all files.
    attributes = ATTR_READ_ONLY | ATTR_ARCHIVE | ATTR_HIDDEN;
	ret = FSFindFirst("*.WAV", attributes, &rec);
	if (ret != 0) return -1;	//nothing found

	if (Check_WAV(rec.filename)==1) 
	{
		WavsInFolder++;
	}
    while (!FSFindNext( &rec ))
    {
		if (Check_WAV(rec.filename)==1)
		{
			WavsInFolder++;
		}
    }
	MusicNum = 1;	//now playing = 1st file
	//Player_GotoMusicHead(MusicNum);

	return WavsInFolder;
}

void read4bytes(FSFILE * f, unsigned char * s)
{

	FSfread(s, 4, 1, f);	
}
unsigned char get_BYTE(FSFILE *f)
{
	unsigned char b;
	FSfread(&b, 1, 1, f);
	return b;
}
unsigned short get_USHORT(FSFILE * f)
{
	unsigned char b0;
	unsigned char b1;
	FSfread(&b0, 1, 1, f);
	FSfread(&b1, 1, 1, f);
	return (unsigned short)b0 + (((unsigned short)b1) << 8);
}

unsigned long get_ULONG(FSFILE *f)
{
	unsigned short us0;
	unsigned short us1;
	us0 = get_USHORT(f);
	us1 = get_USHORT(f);
	return (unsigned long)us0 + (((unsigned long)us1) << 16);
}

short Check_WAV_Header(char * filename)
{
	FSFILE * fo;
	short pos = 0;	//position in Buffer
    unsigned char s[8];
	short i;
	unsigned long temp_l;

	fo = FSfopen(filename, "rb");
	read4bytes(fo, s);
	temp_l = get_ULONG(fo);							// total file length - 8
	read4bytes(fo, s);
	read4bytes(fo, s);
	temp_l = get_ULONG(fo);							// chunk length
	temp_l = get_USHORT(fo);						// format
	temp_l = get_USHORT(fo);	
	temp_l = get_ULONG(fo);	
	temp_l = get_ULONG(fo);							// bytes/sec
	temp_l = get_USHORT(fo);						// Bytes x channels	
	temp_l = get_USHORT(fo);						// bits/sample	

	pos = 36;
	//skip extended chunks
	while (pos < 512)
	{
		//get chunk header
		read4bytes(fo, s);
		pos += 4;
		//get chunk length
		temp_l = get_ULONG(fo);
		pos += 4;
		//if chunk = data, do nothing.
        if (memcmp(s, "data", 4) == 0) break;
		//consume chunk
		pos += (short)temp_l;
		for (i = 0; i < (int)temp_l; i++)
		{
			temp_l = get_BYTE(fo);
		}
	}

	FSfclose(fo);
	return pos;
}

BYTE Check_WAV(char * filename)	//If that is WAV file, return 1 else 0
{

	FSFILE * fo;
	int ret = 1;
	int pos = 0;	//position in Buffer
    unsigned char s[8];
	int i;
	unsigned long temp_l;

	fo = FSfopen(filename, "rb");
	if (fo == NULL) return 0;

	read4bytes(fo, s);
	if (memcmp(s, "RIFF", 4) != 0)  ret = 0;		// start with RIFF
	temp_l = get_ULONG(fo);							// total file length - 8
	read4bytes(fo, s);
	if (memcmp(s, "WAVE", 4) != 0) ret = 0;			// RIFF type is WAVE
	read4bytes(fo, s);
	if (memcmp(s, "fmt ", 4) != 0) ret = 0;			// fmt chunk
	temp_l = get_ULONG(fo);							// chunk length
	if (temp_l != 0x10 ) ret = 0;					// Linear PCM, should be 16 bytes
	temp_l = get_USHORT(fo);						// format
	if (temp_l != 1 )	 ret = 0;					// Linear PCM, 1
	temp_l = get_USHORT(fo);	
	if (temp_l != 2 )	 ret = 0;					// Stereo, 2 ch
	temp_l = get_ULONG(fo);	
	if (temp_l != 44100 )	 ret = 0;				// 44100 sample/sec				
	temp_l = get_ULONG(fo);							// bytes/sec
	if (temp_l != 176400 )	 ret = 0;				// 44100 x 4 bytes /sec				
	temp_l = get_USHORT(fo);						// Bytes x channels	
	if (temp_l != 4 )	 ret = 0;					// 4 byte / sample
	temp_l = get_USHORT(fo);						// bits/sample	
	if (temp_l != 16 )	 ret = 0;					// 16 bit

	if (ret == 0)	//after initial check...
	{
		FSfclose(fo);
		return ret;
	}

	pos = 36;
	//skip extended chunks
	while (pos < 512)
	{
		//get chunk header
		read4bytes(fo, s);
		pos += 4;
		//get chunk length
		temp_l = get_ULONG(fo);
		pos += 4;
		//if chunk = data, do nothing.
        if (memcmp(s, "data", 4) == 0) break;
		//consume chunk
		pos += temp_l;
		for (i = 0; i < (int)temp_l; i++)
		{
			temp_l = get_BYTE(fo);
		}
	}

	FSfclose(fo);
	return ret;
}	

int Player_MusicsInFolder(void)
{
	return WavsInFolder;
}

int Player_MusicNum(void)
{
	return MusicNum;
}

void Player_FolderName(char * dest)
{
	//FSgetcwd, get current folder name
	//#TODO:length?
	memcpy(dest, FSgetcwd(NULL, 10),10);
}

int Player_TotalFolder(void)
{
	return MaxFolderCount;
}

void Player_FolderInfo(char * dest)
{
	SearchRec rec;
	ULONG TotalFileSize = 0;
	short minutes;
	short seconds;
	UINT TotalSecond;
	FSFILE * fo;
    unsigned char attributes;
	//lookup all files.
    attributes = ATTR_READ_ONLY | ATTR_ARCHIVE | ATTR_HIDDEN;
	FSFindFirst("*.WAV", attributes, &rec);

	if (Check_WAV(rec.filename)==1) 
	{
		fo = FSfopen(rec.filename, "rb");
		TotalFileSize += fo->size;
		FSfclose(fo);
	}
    while (!FSFindNext( &rec ))
    {
		if (Check_WAV(rec.filename)==1)
		{
			fo = FSfopen(rec.filename, "rb");
			TotalFileSize += fo->size;
			FSfclose(fo);
		}
    }
	MusicNum = 1;	//now playing = 1st file
	//Player_GotoMusicHead(MusicNum);	//back to 1st file

	TotalSecond = ((TotalFileSize)>>10) / 176;	
	minutes = (UINT)(TotalSecond / 60);
	seconds = TotalSecond % 60;
	sprintf(dest, "%02u %02u:%02u",WavsInFolder, minutes, seconds);
}

void Player_MusicName(char * dest, short RequestMusic)
{
	int ret;
	SearchRec rec;
    unsigned char attributes;
	short togoMusic = 0;

    if (RequestMusic < 1)  RequestMusic = 1;
	if (RequestMusic > WavsInFolder) RequestMusic = WavsInFolder;

    attributes = ATTR_READ_ONLY | ATTR_ARCHIVE | ATTR_HIDDEN;
	//search for specified indexed wav file.
	ret = FSFindFirst("*.WAV", attributes, &rec);
	if (ret != 0) return;	//nothing found

	if (Check_WAV(rec.filename)==1) togoMusic ++;
	
	if (togoMusic < RequestMusic)
	{
		while (!FSFindNext( &rec ))
		{
			if (Check_WAV(rec.filename)==1) togoMusic ++;
			if (togoMusic >= RequestMusic) break;
		}
	}
	//now rec.filename is the wanted file name

	memcpy(dest,  rec.filename, 8);	
}	

//For starting info
void Player_TotalMinSec(char * dest, short RequestMusic)
{
	
	//return mm:ss as Total minutes:second
	UINT TotalSecond;
	UINT Minutes;
	UINT Seconds;
	//0x 00:00 buffer
	char MinSec[10];
	int ret;
	SearchRec rec;
    unsigned char attributes;
	short togoMusic = 0;
	FSFILE * fo;

    if (RequestMusic < 1)  RequestMusic = 1;
	if (RequestMusic > WavsInFolder) RequestMusic = WavsInFolder;

    attributes = ATTR_READ_ONLY | ATTR_ARCHIVE | ATTR_HIDDEN;
	//search for specified indexed wav file.
	ret = FSFindFirst("*.WAV", attributes, &rec);
	if (ret != 0) return;	//nothing found

	if (Check_WAV(rec.filename)==1) togoMusic ++;
	
	if (togoMusic < RequestMusic)
	{
		while (!FSFindNext( &rec ))
		{
			if (Check_WAV(rec.filename)==1) togoMusic ++;
			if (togoMusic >= RequestMusic) break;
		}
	}
	//now rec.filename is the wanted file name

	memset(MinSec,0x00,sizeof(MinSec));
	//open file to check
	fo = FSfopen(rec.filename, "rb");
	TotalSecond = ((fo->size)>>10) / 176;	
	FSfclose(fo);
	Minutes = (UINT)(TotalSecond / 60);
	Seconds = TotalSecond % 60;
	
	sprintf(&MinSec[0],"%02u ", (UINT)(MusicNum & 0x00FF));
	sprintf(&MinSec[3],"%02u", Minutes);
	sprintf(&MinSec[5],":");
	sprintf(&MinSec[6],"%02u", Seconds);
	
	memcpy(dest, MinSec, 8);
}	

//Playing info, erapsed min:sec
void Player_CurrentMinSec(char * dest)
{
	
	//return mm:ss as played minutes:second
	UINT PlayedSeconds = 0;
	UINT Minutes = 0;
	UINT Seconds = 0;
	long position = 0; 
	//0x 00:00 buffer
	char MinSec[10];

	memset(MinSec,0x00,sizeof(MinSec));
	position = FSftell(Currentfo);
	PlayedSeconds = (position>>10) / 176;	
	Minutes = (UINT)(PlayedSeconds / 60);
	Seconds = PlayedSeconds % 60;

	sprintf(&MinSec[0],"%02u ", (UINT)(MusicNum & 0x00FF));
	sprintf(&MinSec[3],"%02u", Minutes);
	sprintf(&MinSec[5],":");
	sprintf(&MinSec[6],"%02u", Seconds);
	
	memcpy(dest, MinSec, 8);
}	


BYTE Player_GotoMusicHead(short RequestMusic)
{
	int i;
	int ret;
	SearchRec rec;
    unsigned char attributes;
	short togoMusic = 0;
	short headsize = 0;

    if (RequestMusic < 1)  RequestMusic = 1;
	if (RequestMusic > WavsInFolder) RequestMusic = WavsInFolder;


    attributes = ATTR_READ_ONLY | ATTR_ARCHIVE | ATTR_HIDDEN;
	//search for specified indexed wav file.
	ret = FSFindFirst("*.WAV", attributes, &rec);
	if (ret != 0) return -1;	//nothing found

	if (Check_WAV(rec.filename)==1) togoMusic ++;
	
	if (togoMusic < RequestMusic)
	{
		while (!FSFindNext( &rec ))
		{
			if (Check_WAV(rec.filename)==1) togoMusic ++;
			if (togoMusic >= RequestMusic) break;
		}
	}
	//now rec.filename is the wanted file name

	MusicNum = RequestMusic;
	fCounter = 0;

	//clear intermediate buffer struct
	InitPlayBackStruct(&pbData);

	headsize = Check_WAV_Header(rec.filename);
	//open file to play
	Currentfo = FSfopen(rec.filename, "rb");
	//read first block*num
	FSfread((void *)pbData.pOutput[0], BLOCKSIZE, BLOCKNUM/2,Currentfo);
	fCounter += BLOCKSIZE * BLOCKNUM/2;
	//fill header area 0
	memset((void *)(pbData.pOutput[0]), 0x00, (size_t)(headsize));

	//now intermediate buffer [0]to[3] is ready.
	for (i = 0; i < BLOCKNUM/2; i++)
	{
		pbData.FilledFlag[i] = 1;
	}

	//Fill spare buffer, 4 to 7.
	FSfread((void *)(pbData.pOutput[BLOCKNUM/2]), BLOCKSIZE, BLOCKNUM/2, Currentfo);
	fCounter += BLOCKSIZE*BLOCKNUM/2;
	for (i = BLOCKNUM/2; i < BLOCKNUM; i++)
	{
		pbData.FilledFlag[i] = 1;
	}
	return 1;
}	

void Player_Open(void)
{
	playdevice_dma0_start(&pbData);	//restart DMA
}

void Player_Close(void)
{
    playdevice_dma0_stop();
}

void Player_Stop(void)
{
	//Stop SPDIF Stream
	playdevice_stop();
	initplaydeviceBuff(&pbData);
    InitPlayBackStruct(&pbData);	
	FSfclose(Currentfo);
    fCounter = 0;
}
void Player_Start(void)
{
	//start SPDIF Stream
	playdevice_start();
}	

void Player_Pause(void)
{
	//SPDIF Stream, intermediate buffer, are set to all zero.
	initplaydeviceBuff(&pbData);
	//DMA or portAudio: keep running, all 0 sent
}

int Player_Play(void)
{
	int i = 0;
	int j = 0;
	int flagcheck = 0;
	int ToContinue = 1;	//initially, continue mode

	if (fCounter > Currentfo->size - BLOCKSIZE*BLOCKNUM/2) 
    {
        ToContinue = 0;
    }
    else   
	{	
		for (i = 0; i < 2; i++)
		{
			flagcheck = 0;
			for (j = 0; j < BLOCKNUM/2; j++)
			{
				flagcheck += pbData.FilledFlag[BLOCKNUM/2*i + j];
			}
			
			if (flagcheck == 0) //Buffer to be filled
			{
				FSfread((void *)(pbData.pOutput[BLOCKNUM/2*i]), BLOCKSIZE, BLOCKNUM/2, Currentfo);
				fCounter += BLOCKSIZE*BLOCKNUM/2;
				//set filled flag
				for (j = 0; j < BLOCKNUM/2; j++)
				{
					pbData.FilledFlag[BLOCKNUM/2*i + j] = 1;
				}

			}
		}
	}
	return ToContinue;
}	


void Player_FF(void)
{
	//go to next music
	MusicNum += 1;
	if (MusicNum > WavsInFolder )
	{
		MusicNum = WavsInFolder;
	}
}

void Player_Prev(void)
{
	//go to previous music
	MusicNum -= 1;
	if (MusicNum < 1) 
	{
		MusicNum = 1;
	}
}	

void Player_CDFF(void)
{
	FSchdir("\\"); //back to root.
	CurrentFolderIndex += 1;
	if (CurrentFolderIndex > MaxFolderCount )
	{
		CurrentFolderIndex = 1;
	}

	//step search to next CD, if max go back
	while( (Player_CD(CurrentFolderIndex) == 0) && (CurrentFolderIndex < MaxFolderCount))
	{
		CurrentFolderIndex += 1;
		if (CurrentFolderIndex > MaxFolderCount) 
		{
			CurrentFolderIndex = MaxFolderCount;
			Player_CD(CurrentFolderIndex);
			break;
		}
		else
		{
			Player_CD(CurrentFolderIndex);
		}
	}

	return;
}
void Player_CDPrev(void)
{
	FSchdir("\\"); //back to root.
	CurrentFolderIndex -= 1;
	if (CurrentFolderIndex < 1 )
	{
		CurrentFolderIndex = MaxFolderCount;
	}

	while( (Player_CD(CurrentFolderIndex) == 0) && (CurrentFolderIndex > 1))
	{
		CurrentFolderIndex -= 1;
		if (CurrentFolderIndex < 1) 
		{
			CurrentFolderIndex = 1;
			Player_CD(CurrentFolderIndex);
			break;
		}
		else
		{
			Player_CD(CurrentFolderIndex);
		}
	}

	return;
}

BYTE Player_CD(int togofolder)
{
	BYTE ret = 0;
	SearchRec rec;
	short togocount = 0;

	if (togofolder > MaxFolderCount ) togofolder = MaxFolderCount;
	if (togofolder < 1 ) togofolder = 1;

	FSchdir("\\");	//ch to root

	//lookup every folder.
	ret = FSFindFirst("*", ATTR_DIRECTORY, &rec);
	if (ret != 0) return -1;	//DISK ERROR?

	//First Folder found. trace every folder.
	ret = Player_FolderInit(rec.filename);
	FSchdir("\\");	//ch to root
	togocount++;	//togocount should be 1. (first folder)
	if (togocount < togofolder)
	{
		while (!FSFindNext( &rec ))
		{
			ret = Player_FolderInit(rec.filename);
			FSchdir("\\");	//ch to root
			togocount++;
			if (togocount >= togofolder) break;
		}
	}
	if (ret > 0)
	{
		//exit loop now, and rec.filename is to go folder name.
		FSchdir(rec.filename);	//ch to togo folder. this folder has wav file
		return 1;
	}
	else return ret;

	//after this function succeed: in target folder, MaxMusics initialized
}





