#ifndef __PC__
#include <p33FJ64GP306.h>
#endif
#include "delay.h"
//#include "SPI_SD.h"
#include "SD-SPI.h"
#include "integer.h"

// This source handles SPI, and SD Card Command level.
// File structure is managed by upper layer.

#ifndef __PC__
#define	SD_CS	PORTGbits.RG9
#endif

//prototype internal
void SD_CLK(void);
BYTE SD_CardCommand(BYTE cmd, LONG Argument);
BYTE SPI_read(BYTE flag);
void SPI_write(BYTE data, BYTE flag);
BYTE SD_Read_Single_Block_Phase1(LONG Address);

BYTE temp;

void MDD_SDSPI_InitIO(void)
{
#ifndef __PC__
	SD_CS = 1;
	TRISGbits.TRISG9 = 0;
	
	// setup the SPI peripheral
	SPI2STAT = 0x0;  // disable the SPI module (just in case)
	SPI2CON1 = 0x013E;
	// bit 12 DISSCK = 0. SCK enabled.
	
	// bit 11 DISSDO = 0. SDO enabled.
	// bit 10 MODE16 = 0. Byte(8bit) Mode.
	// bit 9  SMP = 0.    input sampled at middle of data output time.
	// bit 8  CKE = 1.    SPI Clock Edge is active to idle.
	
	// bit 7  SSEN = 0.   slave select is disabled.(master mode)
	// bit 6  CKP = 0.    Clock Polarity is, High/Active, Low/Idle.
	// bit 5  MSTEN = 1.  Master Mode.
	// bit 4-2  SPRE=111. Second Prescale bit = 1:1.
	// bit 1-0  PPRE=10.  Primary Prescale = 4:1.
	// SCK = Fcy 33.864MHz / 4 / 1 = 8.47MHz
	SPI2STAT = 0x8000; // enable the SPI module

#endif
	//Wait SD Card
	Delay_Us(20);
	SD_CLK();
	Delay_Us(20);
	
}

//send CLOCK for SD card, let it sample clock
void SD_CLK(void)
{
#ifndef __PC__
	int i;
	BYTE ret;
	SD_CS = 0;
	Delay8Tcy();
	for (i = 0; i < 100; i++)
	{
		ret = SPI_read(0);
	}
	SD_CS = 1;
#endif
}	

BYTE MDD_SDSPI_MediaInitialize(void)
{
#ifndef __PC__
	BYTE stat, ret[4];
	DWORD Count;
	
	Count = 0;
	SD_CS = 0;  //CS
	Delay8Tcy();
    //send dummy clocks
    for (Count = 0; Count < 100; Count++) stat = SPI_read(0);
    Count = 0;

	// index 0: GO_IDLE_STATE command
	stat = SD_CardCommand(0, 0);

	// repeat until get 0x01 (In Idle State) answer.
	while( (stat != 1) )
	{
		stat = SD_CardCommand(0, 0);
		Count++;
		if ( Count > 8000)
		{
			return FALSE;	//failed
		}
	}
    Count = 0;

    //check SDHC or SD/MMC
    stat = SD_CardCommand(8, 0x000001AA);    //CMD 8 VHS 1(2.7-3.6) pattern 10101010b
    if (stat == 1)
    {
        //Ver 2.00 mode initialization
		ret[0] = SPI_read(0);
		ret[1] = SPI_read(0);
		ret[2] = SPI_read(0);
		ret[3] = SPI_read(0);  //read R7 response
        //not checking 3.3V oparational... 

        //send ACMD41
        stat = TRUE;
        Count = 0;

    	while( stat )   //if idle stat == 0x01
    	{
            //need to send CMD55 before ACMD
            SD_CardCommand(55, 0);
            //ACMD 41 with bit 30, High Capacity = 1
    		stat = SD_CardCommand(41, 0x40000000);
    		Count++;
        	Delay8Tcy();
    		if ( Count > 1000)
    		{
                SD_CS = 1;
    			return FALSE;	//failed
    		}
    	}
     
    }
    else
    {
        // SD (ver 1) or MMC.
    	// index 1: SEND_OP_COMMAND, card initializing
    	// repeat until get 0x00 (In Idle State bit cleared).
    	while ( stat == 1 )
    	{
    		Delay_Us(1);
    		stat = SD_CardCommand(1, 0);
    	}
    }

	SD_CS = 1;
#endif
	return TRUE;
}	

BYTE MDD_SDSPI_WriteProtectState(void)
{
    return(TRUE);
}

BYTE MDD_SDSPI_SectorRead(DWORD Address, BYTE * Buffer)
{
	BYTE stat;
	BYTE ret;
	int i;

	stat = 0;
	ret = TRUE;
	i = 0;
#ifndef __PC__
	SD_CS = 0;
	
	//Send command (single read) and wait response
	stat = SD_Read_Single_Block_Phase1(Address);
	
	if (stat == 0)
	{
		//store return value to buffer
		for(i = 0; i < 512; i++)
		{
			*Buffer = SPI_read(0);
			Buffer++;	
		}
		ret = TRUE;

		stat = SPI_read(0);
		stat = SPI_read(0);
		
		//Check transaction finished
		for (i = 0; i < 800; i++)
		{
			stat = SPI_read(0);
			if (stat == 0xFF) break;
		}
	}
	else
	{ ret = FALSE; }
	SD_CS = 1;
#endif

	return ret;			
}	

BYTE SD_Read_Single_Block_Phase1(LONG Address)
{
	int RetryCount;
	BYTE ret;
	int i;
	
	ret = 0;

	for (RetryCount = 0; RetryCount < 100; RetryCount++)
	{
		ret = SD_CardCommand(17, Address);
       
		if (ret == 0)
		{
			//Not ready. wait for Data token.
			for (i = 0; i < 5000; i++)
			{
				ret = SPI_read(0);
				
				if (ret == 0xFE) return 0;	//got token
			}	
		}
		else if (ret == 0xFE)
		{
			//Got Token.
			return 0;
		}
	}
		
	return 1; // can not opened.
}	


// Initialize SD Card to SPI Mode
BYTE SD_CardCommand(BYTE cmd, LONG Argument)
{
    //cmd: 0,1,... 
	BYTE ret;
    BYTE CRC;
	int i;

	//Confirm card nop, before send command
	// if no command sent, should return 0xFF.
	for (i = 0; i < 800; i++)
	{
		ret = SPI_read(0);
		if (ret == 0xFF) 
		{
			break;
		}
	}

	// failed. upper layer will handle.
	if (ret != 0xFF) return 0;

	// SD Card Command	
   	SPI_write(0x40 | cmd,0);	//command, 0x40 + index
	SPI_write((BYTE)(Argument >> 24) & 0xFF,0);	//Argument 0
	SPI_write((BYTE)(Argument >> 16) & 0xFF,0);	//Argument 1
	SPI_write((BYTE)(Argument >>  8) & 0xFF,0);	//Argument 2
	SPI_write((BYTE)(Argument      ) & 0xFF,0);		//Argument 3
    
    CRC = 0x01;
    if (cmd == 8)
    {
    	CRC = 0x87;	//CRC for CMD8 SEND_IF_COND.
    }
    if (cmd == 0)
    {
    	CRC = 0x95;	//CRC for GO_STATE_IDLE only. others, don't care
    }
    SPI_write(CRC,0);
	
    // Some command take long time to answer.
	for (i = 0; i < 512; i++)
	{
		ret = SPI_read(0);
		if (ret == 0xFE) 
		{
			break;	//data token
		}
		if ( (ret & 0x80) == 0x00 )
		{
			break; 	//got response
		}
	}

	return ret;	
}	


void SPI_write(BYTE data, BYTE flag)
// data: to send. flag(SS): not used.
{
#ifndef __PC__
	temp = SPI2BUF;			// dummy read of the SPI1BUF register
							// to clear the SPIRBF flag
	SPI2BUF = data;			// write the data out to the SPI peripheral
	
	while(!SPI2STATbits.SPIRBF)
	{
		Nop(); 
	}
#endif
}

BYTE SPI_read(BYTE flag)
// retrun: read value. flag(SS): not used.
{
#ifndef __PC__
	temp = SPI2BUF;			// dummy read of the SPI1BUF register
							// to clear the SPIRBF flag
	SPI2BUF = 0xFF;			// write the data out to the SPI peripheral
		
	while(!SPI2STATbits.SPIRBF)
	{
		Nop(); 
	}
	
	temp = 0xFF & SPI2BUF;			// read result

#endif
	return temp;	
}	

