// Button.C
// handles button IO routine.
#ifndef __PC__
#include <p33FJ64GP306.h>
//#include "generictypedefs.h"
#else
#include	<stdio.h>	//read control file
#endif

#include "integer.h"	//BYTE etc

#include "button.h"		//function, ENUM prototype

//Current value, Prev Value for buttons
BYTE ButtonStatus_Stop;
BYTE ButtonPrev_Stop;
BYTE ButtonStatus_Pause;
BYTE ButtonPrev_Pause;
BYTE ButtonStatus_Play;
BYTE ButtonPrev_Play;
BYTE ButtonStatus_FF;
BYTE ButtonPrev_FF;
BYTE ButtonStatus_Prev;
BYTE ButtonPrev_Prev;
BYTE ButtonStatus_CDFF;
BYTE ButtonPrev_CDFF;
BYTE ButtonStatus_CDPrev;
BYTE ButtonPrev_CDPrev;


//PORT B Connection
// 7:CDPREV
// 6:CDFF
// 5:PLAY
// 4:PAUSE
// 3:STOP
// 2:
// 1:FF
// 0:PREV

void Init_button(void)
{
	// Set B0,1,2,3,4,5, 6, 7 as input
	// This is initial condition but make it sure.
#ifndef __PC__
	TRISB = TRISB | 0x00FF;
	
	// AD1PCFGL: Change A/D Port.
	// AN15 - AN0 is not used, connected to AVss.
	// Digital Port use these pins. 
	AD1PCFGL = 0xFFFF;	
#endif	
	// Initialize button variables
	ButtonStatus_Stop = 0;
	ButtonPrev_Stop = 0;
	ButtonStatus_Pause = 0;
	ButtonPrev_Pause = 0;
	ButtonStatus_Play = 0;
	ButtonPrev_Play = 0;
	ButtonStatus_FF = 0;
	ButtonPrev_FF = 0;
	ButtonStatus_Prev = 0;
	ButtonPrev_Prev = 0;
	ButtonStatus_CDFF = 0;
	ButtonPrev_CDFF = 0;
	ButtonStatus_CDPrev = 0;
	ButtonPrev_CDPrev = 0;
}

void Read_Buttons(void)
{
	// called by timer function, _T2Interrupt
	BYTE StopBtn;
	BYTE PauseBtn;
	BYTE PlayBtn;
	BYTE FFBtn;
	BYTE PrevBtn;
	BYTE CDFFBtn;
	BYTE CDPrevBtn;
#ifndef __PC__	
	StopBtn = PORTBbits.RB3;
	PauseBtn = PORTBbits.RB4;
	PlayBtn = PORTBbits.RB5;
	FFBtn = PORTBbits.RB1;
	PrevBtn = PORTBbits.RB0;
	CDFFBtn = 0; //PORTBbits.RB6;
	CDPrevBtn = 0; //PORTBbits.RB7;
#else
	FILE *fp;
	int ch;
	StopBtn = 0;
	PauseBtn = 0;
	PlayBtn = 0;
	FFBtn = 0;
	PrevBtn = 0;
	CDFFBtn = 0;
	CDPrevBtn = 0;
	if(fp = fopen("c:\\temp\\keypad.txt","r"))
	{
		ch = fgetc(fp);
		if (ch > '0') printf("Key %d\r",ch);

		if (ch == '1') StopBtn = 1;
		if (ch == '2') PauseBtn = 1;
		if (ch == '3') PlayBtn = 1;
		if (ch == '4') FFBtn = 1;
		if (ch == '5') PrevBtn = 1;
		if (ch == '6') CDFFBtn = 1;
		if (ch == '7') CDPrevBtn = 1;
		fclose(fp);
	}
#endif	
	if (StopBtn == ButtonPrev_Stop)
	//  When "Same as before" value sampled, to avoid < 10ms noise
	{
		ButtonStatus_Stop = StopBtn;	//store into current value.
	}
	//save 'value got now' as Prev value, to compare at next timer cycle.
	ButtonPrev_Stop = StopBtn;
	
	if (PauseBtn == ButtonPrev_Pause)
	{
		ButtonStatus_Pause = PauseBtn;
	}
	ButtonPrev_Pause = PauseBtn;
	
	if (PlayBtn == ButtonPrev_Play)
	{
		ButtonStatus_Play = PlayBtn;
	}
	ButtonPrev_Play = PlayBtn;
	
	if (FFBtn == ButtonPrev_FF)
	{
		ButtonStatus_FF = FFBtn;
	}
	ButtonPrev_FF = FFBtn;
	
	if (PrevBtn == ButtonPrev_Prev)
	{
		ButtonStatus_Prev = PrevBtn;
	}
	ButtonPrev_Prev = PrevBtn;
	if (CDFFBtn == ButtonPrev_CDFF)
	{
		ButtonStatus_CDFF = CDFFBtn;
	}
	ButtonPrev_CDFF = CDFFBtn;
	
	if (CDPrevBtn == ButtonPrev_CDPrev)
	{
		ButtonStatus_CDPrev = CDPrevBtn;
	}
	ButtonPrev_CDPrev = CDPrevBtn;}	

BYTE Read_Command()
{
	//will be called from  main().
	//return button code with priority.
	BYTE retval;
	retval = BTN_NONE;
	//for debug: Read_Buttons();
	// Priority high to low.
	
	if (ButtonStatus_Stop)
	{ 
		retval = BTN_STOP;
	}

	if ( ( retval == BTN_NONE ) && ButtonStatus_Pause)
	{
		retval = BTN_PAUSE;
	}

	if ( ( retval == BTN_NONE ) && ButtonStatus_Play)
	{
		retval = BTN_PLAY;
	}

	if ( ( retval == BTN_NONE ) && ButtonStatus_FF) 
	{
		retval = BTN_FF;
	}

	if ( ( retval == BTN_NONE ) && ButtonStatus_Prev) 
	{
		retval = BTN_PREV;
	}
	if ( ( retval == BTN_NONE ) && ButtonStatus_CDFF) 
	{
		retval = BTN_CDFF;
	}

	if ( ( retval == BTN_NONE ) && ButtonStatus_CDPrev) 
	{
		retval = BTN_CDPREV;
	}

	return retval;
}	
