// main routine for Ultimate Source

#include <string.h>
#ifndef __PC__
#include <p33FJ64GP306.h>
#else
#include <stdio.h>
#endif
#include "lcd.h"	//LCD character output routines
#include "delay.h"	//utility delay routines
#include "button.h"	//button initialize, read
#include "CPU.h"	//Oscilator
#include "Timer.h"	//Timer initialize and interrupt
#include "Player.h"	//Player Module routines
#include "playerhardware.h"	//DMA and control DCI (for PC portaudio)
#include "integer.h"

//Current operation mode
#define MODE_STOP 1
#define MODE_PAUSE 2
#define MODE_PLAY 3

//typedef enum {MODE_STOP, MODE_PAUSE, MODE_PLAY} STATE;


//Timer routine interrupt flag: not used now
BYTE Timer_Flag;

BYTE PlayMode;

//Total musics in Folder. 
int maxMusics;

//Initialize IO
BYTE IoInit(void)
{
	BYTE ret = 0;

	Init_cpu();					//init Clock system
	Delay_ms(100);
	Init_Timer();				//timer interrupt
	Delay_Us(10);
	Init_lcd();					//init LCD
	Delay_ms(100);
	Init_button();				//init button
	Delay_ms(100);

	lcd_str("KDA-US01", 1);		//banner
	Delay_ms(10);
	lcd_str("STARTUP", 2);

	Delay_ms(200);
	
	return ret;
}

void PrintFolder(void)
{
	char foldername[10];
	char folderinfo[10];
	memset(foldername, 0, sizeof(foldername));
	memset(folderinfo, 0, sizeof(folderinfo));
	//Get Folder name
	Player_FolderName(foldername);
	//Print File Name at row 1
	Delay_Us(100);	
	lcd_str(foldername, 1);
	//Print XX xx:xx at row 2. Total wavs, total play mm:ss
	Player_FolderInfo(folderinfo);
	lcd_str(folderinfo, 2);
}

void PrintInfo(short RequestMusic)
{
	//DWORD FileSize;
	char musicname[10];
	char PlayMinSec[10];
	memset(musicname, 0, sizeof(musicname));
	memset(PlayMinSec, 0, sizeof(PlayMinSec));
	//Get File name
	Player_MusicName(musicname, RequestMusic);
	//Print File Name at row 1
	Delay_Us(100);	
	lcd_str(musicname, 1);
	Delay_Us(100);	
	//Print 00:00 at row 2
	Player_TotalMinSec(PlayMinSec, RequestMusic);
	lcd_str(PlayMinSec, 2);
}	

void PrintTime(void)
{
	char PlayMinSec[10];
	//Print 00:00 at row 2
	Player_CurrentMinSec(PlayMinSec);
	lcd_str2(PlayMinSec, 2, 3);
}	

int main ( void )
{
	short ret;
	BYTE BtnState;
	BYTE BtnStatePrev;
	BYTE ContinueMode; //1: play continue, 0:music end
	short CurrentMusicNum;
    short TimerCounter = 0;
	short i;

	CurrentMusicNum = 1;
	BtnState = 0x00;
	BtnStatePrev = 0x00;
	Timer_Flag = 0;
	ContinueMode = 1;

	ret = IoInit();

	lcd_str("Loading.", 2);

	ret = Player_Init();		//initialize player
	if (ret == -2)
	{
		lcd_str("NO MUSIC", 2);
		while(1);
	}
	if (ret == -3)
	{
		lcd_str("DISK ERR", 2);
		while(1);
	}
	if (ret == -4)
	{
		lcd_str("CHD ERR", 2);
		while(1);
	}
	if (ret == -5)
	{
		lcd_str("Find ERR", 2);
		while(1);
	}
	
	PlayMode = MODE_STOP;

	//move to CD 1.

	i = 1;
	while(Player_CD(i) == 0)
	{	//until folder contains wav file
		i++;
		if (i > Player_TotalFolder()) 
		{
			lcd_str("FOLDERERR", 2);
			while(1);
		}
	}

	//Print 1st CD info
	PrintFolder();

	//delay for a while
	Delay_ms(1500);

	//Print 1st WAV info
	PrintInfo(1);

	Player_GotoMusicHead(1);
	
	Player_Open();

	//This is Test Pin
#ifndef __PC__
	PORTFbits.RF3 = 0;
	TRISFbits.TRISF3 = 0;
	PORTFbits.RF3 = 1;
#endif

	while(1)
	{
		Delay8Tcy();

		//PLAYER CONTROL
		//Music Stream processing. should be processed priority.
		if (PlayMode == MODE_PLAY)
		{
			ContinueMode = Player_Play();
		}
		
		if (ContinueMode == 0)  //Player retured, indicates end of music
		{
			switch (PlayMode)
			{
				case MODE_STOP:
					break;
				case MODE_PLAY:
					//Get Music num
					CurrentMusicNum = Player_MusicNum();
					//Player played till end of music.
					Player_Stop();	//stop playing, close file
					Delay_Us(200);
					if (CurrentMusicNum < Player_MusicsInFolder())
					{
						Player_FF();    //Player_FF increments MusicNum
    					CurrentMusicNum = Player_MusicNum();
					}
					else
					{
                        //all track played. go to next CD.
						Player_CDFF();	//next folder
						PrintFolder();	//print folder info
						Delay_ms(1500);
                        CurrentMusicNum = 1;
					}
					PrintInfo(CurrentMusicNum);
					Delay_ms(1000);
					Player_GotoMusicHead(CurrentMusicNum);
                	Player_Open();
					Player_Start();
					break;
				case MODE_PAUSE:
					break;
			}
            ContinueMode = 1;
		}
		
		//DISPLAY CONTROL
#ifndef __PC__
		if (Timer_Flag == 1)	
		{
#endif
			//Timer Expired,
            TimerCounter++;
            if (TimerCounter > 120)
            {
                if (PlayMode == MODE_PLAY) PrintTime();
                TimerCounter = 0;
            }
#ifndef __PC__
			Timer_Flag = 0;
		}
#endif
		
		//Button and State CONTROL .. Human interface
		BtnState = Read_Command();
		if (BtnState != BtnStatePrev)
		{
			switch(BtnState)
			{
				case BTN_STOP:
					ContinueMode = 1;
					PlayMode = MODE_STOP;
					Player_Pause();	//clear buffer
					Player_Stop();	//stop playing, close file
					PrintFolder();	//print folder info
					Delay_ms(1500);
                	CurrentMusicNum = 1;
					PrintInfo(CurrentMusicNum);
					Player_GotoMusicHead(CurrentMusicNum);
					//Delayed_Stop = 1;
					break;
				case BTN_PAUSE:
					PlayMode = MODE_PAUSE;
					lcd_str("PAUSE", 2);
					Player_Pause();	//clear buffer
					break;
				case BTN_PLAY:
					PlayMode = MODE_PLAY;
					Player_Pause();	//clear buffer
                    Player_Open();
					Player_Start();
					ContinueMode = 1;
					Player_Play();
					//PrintInfo(CurrentMusicNum);
					break;
				case BTN_FF:
					Player_Pause();	//clear buffer
					Player_Stop();	//stop playing, close file
					Player_FF();
					CurrentMusicNum = Player_MusicNum();
					PrintInfo(CurrentMusicNum);
					Player_GotoMusicHead(CurrentMusicNum);
					break;
				case BTN_PREV:
					Player_Pause();	//clear buffer
					Player_Stop();	//stop playing, close file
					Player_Prev();
					CurrentMusicNum = Player_MusicNum();
					PrintInfo(CurrentMusicNum);
					Player_GotoMusicHead(CurrentMusicNum);
					break;
				case BTN_CDFF:
					Player_Pause();	//clear buffer
					Player_Stop();	//stop playing, close file
					Player_CDFF();	//next folder
					PrintFolder();	//print folder info
					Delay_ms(1500);
                	CurrentMusicNum = 1;
					PrintInfo(CurrentMusicNum);
					Player_GotoMusicHead(CurrentMusicNum);
					break;
				case BTN_CDPREV:
					Player_Pause();	//clear buffer
					Player_Stop();	//stop playing, close file
					Player_CDPrev();	//prev folder
					PrintFolder();	//print folder info
					Delay_ms(1500);
                	CurrentMusicNum = 1;
					PrintInfo(CurrentMusicNum);
					Player_GotoMusicHead(CurrentMusicNum);
					break;
				case BTN_NONE:
					break;	
				default:
					break;	
			}//switch
			BtnStatePrev = BtnState;
		}//if (btnState
	}//while()
}
